@@ -1,7 +1,3 @@
-#ifndef lint
-static char *RCSid() { return RCSid("$Id: misc.c,v 1.188.2.6 2014/10/07 21:22:56 sfeam Exp $"); }
-#endif
-
 /* GNUPLOT - misc.c */
 
 /*[
@@ -38,6 +34,8 @@ static char *RCSid() { return RCSid("$Id: misc.c,v 1.188.2.6 2014/10/07 21:22:56
 
 #include "alloc.h"
 #include "command.h"
+#include "datablock.h"
+#include "encoding.h"
 #include "graphics.h"
 #include "plot.h"
 #include "tables.h"
@@ -45,21 +43,15 @@ static char *RCSid() { return RCSid("$Id: misc.c,v 1.188.2.6 2014/10/07 21:22:56
 #include "variable.h"
 #include "axis.h"
 #include "scanner.h"		/* so that scanner() can count curly braces */
-#ifdef _Windows
+#include "setshow.h"
+#ifdef _WIN32
 # include <fcntl.h>
-# if defined(__WATCOMC__) || defined(__MSC__)
+# if defined(__WATCOMC__) || defined(_MSC_VER)
 #  include <io.h>        /* for setmode() */
 # endif
 #endif
-#if defined(HAVE_DIRENT_H)
-# include <sys/types.h>
-# include <dirent.h>
-#elif defined(_Windows)
-# include <windows.h>
-#endif
 
-static char *recursivefullname __PROTO((const char *path, const char *filename, TBOOLEAN recursive));
-static void prepare_call __PROTO((int calltype));
+static void prepare_call(int calltype);
 
 /* State information for load_file(), to recover from errors
  * and properly handle recursive load_file calls
@@ -71,78 +63,28 @@ int call_argc;
 char *call_args[10] = { NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL, NULL };
 static char *argname[] = {"ARG0","ARG1","ARG2","ARG3","ARG4","ARG5","ARG6","ARG7","ARG8","ARG9"};
 
-/*
- * iso_alloc() allocates a iso_curve structure that can hold 'num'
- * points.
- */
-struct iso_curve *
-iso_alloc(int num)
-{
-    struct iso_curve *ip;
-    ip = (struct iso_curve *) gp_alloc(sizeof(struct iso_curve), "iso curve");
-    ip->p_max = (num >= 0 ? num : 0);
-    ip->p_count = 0;
-    if (num > 0) {
-	ip->points = (struct coordinate GPHUGE *)
-	    gp_alloc(num * sizeof(struct coordinate), "iso curve points");
-	memset(ip->points, 0, num * sizeof(struct coordinate));
-    } else
-	ip->points = (struct coordinate GPHUGE *) NULL;
-    ip->next = NULL;
-    return (ip);
-}
-
-/*
- * iso_extend() reallocates a iso_curve structure to hold "num"
- * points. This will either expand or shrink the storage.
- */
-void
-iso_extend(struct iso_curve *ip, int num)
-{
-    if (num == ip->p_max)
-	return;
-
-    if (num > 0) {
-	if (ip->points == NULL) {
-	    ip->points = (struct coordinate GPHUGE *)
-		gp_alloc(num * sizeof(struct coordinate), "iso curve points");
-	} else {
-	    ip->points = (struct coordinate GPHUGE *)
-		gp_realloc(ip->points, num * sizeof(struct coordinate), "expanding curve points");
-	}
-	if (num > ip->p_max)
-	    memset( &(ip->points[ip->p_max]), 0, (num - ip->p_max) * sizeof(struct coordinate));
-	ip->p_max = num;
-    } else {
-	if (ip->points != (struct coordinate GPHUGE *) NULL)
-	    free(ip->points);
-	ip->points = (struct coordinate GPHUGE *) NULL;
-	ip->p_max = 0;
-    }
-}
-
-/*
- * iso_free() releases any memory which was previously malloc()'d to hold
- *   iso curve points.
- */
-void
-iso_free(struct iso_curve *ip)
-{
-    if (ip) {
-	if (ip->points)
-	    free((char *) ip->points);
-	free((char *) ip);
-    }
-}
+/* Used by postscript terminal if a font file is found by loadpath_fopen() */
+char *loadpath_fontname = NULL;
 
 static void
 prepare_call(int calltype)
 {
     struct udvt_entry *udv;
+    struct value *ARGV;
     int argindex;
+    int argv_size;
+
+    /* call_args[] will hold arguments as strings.
+     * argval[] will be a private copy of numeric arguments as udvs.
+     * Later we will fill ARGV[] from one or the other.
+     */
+    struct value argval[9];
+    for (argindex = 0; argindex < 9; argindex++)
+	argval[argindex].type = NOTDEFINED;
+
     if (calltype == 2) {
 	call_argc = 0;
-	while (!END_OF_COMMAND && call_argc <= 9) {
+	while (!END_OF_COMMAND && call_argc < 9) {
 	    call_args[call_argc] = try_to_get_string();
 	    if (!call_args[call_argc]) {
 		int save_token = c_token;
@@ -152,11 +94,15 @@ prepare_call(int calltype)
 		    call_args[call_argc] = gp_strdup(add_udv(c_token)->udv_value.v.string_val);
 		    c_token++;
 
-		/* Evaluates a parenthesized expression and store the result in a string */
-		} else if (equals(c_token, "(")) {
+		/* Evaluate a parenthesized expression or a bare numeric user variable
+		 * and store the result in a string
+		 */
+		} else if (equals(c_token, "(")
+			|| (type_udv(c_token) == INTGR || type_udv(c_token) == CMPLX)) {
 		    char val_as_string[32];
 		    struct value a;
 		    const_express(&a);
+		    argval[call_argc] = a;
 		    switch(a.type) {
 			case CMPLX: /* FIXME: More precision? Some way to provide a format? */
 				sprintf(val_as_string, "%g", a.v.cmplx_val.real);
@@ -165,17 +111,23 @@ prepare_call(int calltype)
 			default:
 				int_error(save_token, "Unrecognized argument type");
 				break;
-			case INTGR:	
-				sprintf(val_as_string, "%d", a.v.int_val);
+			case INTGR:
+				sprintf(val_as_string, PLD, a.v.int_val);
 				call_args[call_argc] = gp_strdup(val_as_string);
 				break;
-		    } 
+		    }
 
-		/* old (pre version 5) style wrapping of bare tokens as strings */
-		/* is still useful for passing unquoted numbers */
+		/* Old (pre version 5) style wrapping of bare tokens as strings
+		 * is still used for storing numerical constants ARGn but not ARGV[n]
+		 */
 		} else {
+		    double temp;
+		    char *endptr;
 		    m_capture(&call_args[call_argc], c_token, c_token);
 		    c_token++;
+		    temp = strtod(call_args[call_argc], &endptr);
+		    if (endptr != call_args[call_argc] && *endptr == '\0')
+			Gcomplex(&argval[call_argc], temp, 0.0);
 		}
 	    }
 	    call_argc++;
@@ -197,22 +149,39 @@ prepare_call(int calltype)
 	call_argc = 0;
     }
 
-    /* Old-style "call" arguments were referenced as $0 ... $9 and $# */
-    /* New-style has ARG0 = script-name, ARG1 ... ARG9 and ARGC */
-    /* FIXME:  If we defined these on entry, we could use get_udv* here */
+    /* Old-style "call" arguments were referenced as $0 ... $9 and $#.
+     * New-style has ARG0 = script-name, ARG1 ... ARG9 and ARGC
+     * Version 5.3 adds ARGV[n]
+     */
     udv = add_udv_by_name("ARGC");
     Ginteger(&(udv->udv_value), call_argc);
-    udv->udv_undef = FALSE;
+
     udv = add_udv_by_name("ARG0");
     gpfree_string(&(udv->udv_value));
     Gstring(&(udv->udv_value), gp_strdup(lf_head->name));
-    udv->udv_undef = FALSE;
+
+    udv = add_udv_by_name("ARGV");
+    free_value(&(udv->udv_value));
+
+    argv_size = GPMIN(call_argc, 9);
+    udv->udv_value.type = ARRAY;
+    ARGV = udv->udv_value.v.value_array = gp_alloc((argv_size + 1) * sizeof(t_value), "array state");
+    ARGV[0].v.int_val = argv_size;
+    ARGV[0].type = NOTDEFINED;
+
     for (argindex = 1; argindex <= 9; argindex++) {
-	char *arg = gp_strdup(call_args[argindex-1]);
+	char *argstring = call_args[argindex-1];
+
 	udv = add_udv_by_name(argname[argindex]);
 	gpfree_string(&(udv->udv_value));
-	Gstring(&(udv->udv_value), arg ? arg : gp_strdup(""));
-	udv->udv_undef = FALSE;
+	Gstring(&(udv->udv_value), argstring ? gp_strdup(argstring) : gp_strdup(""));
+
+	if (argindex > argv_size)
+	    continue;
+	if (argval[argindex-1].type == NOTDEFINED)
+	    Gstring(&ARGV[argindex], gp_strdup(udv->udv_value.v.string_val));
+	else
+	    ARGV[argindex] = argval[argindex-1];
     }
 }
 
@@ -276,12 +245,13 @@ expand_call_args(void)
 #endif /* OLD_STYLE_CALL_ARGS */
 
 /*
- * load_file() is called from
+ * calltype indicates whether load_file() is called from
  * (1) the "load" command, no arguments substitution is done
  * (2) the "call" command, arguments are substituted for $0, $1, etc.
  * (3) on program entry to load initialization files (acts like "load")
  * (4) to execute script files given on the command line (acts like "load")
  * (5) to execute a single script file given with -c (acts like "call")
+ * (6) "load $datablock" (EXPERIMENTAL)
  */
 void
 load_file(FILE *fp, char *name, int calltype)
@@ -291,19 +261,22 @@ load_file(FILE *fp, char *name, int calltype)
     int start, left;
     int more;
     int stop = FALSE;
+    udvt_entry *gpval_lineno = NULL;
+    char **datablock_input_line = NULL;
+
+    /* Support for "load $datablock" */
+    if (calltype == 6)
+	datablock_input_line = get_datablock(name);
+
+    if (!fp && !datablock_input_line)
+	int_error(NO_CARET, "Cannot load input from '%s'", name);
 
     /* Provide a user-visible copy of the current line number in the input file */
-    udvt_entry *gpval_lineno = add_udv_by_name("GPVAL_LINENO");
+    gpval_lineno = add_udv_by_name("GPVAL_LINENO");
     Ginteger(&gpval_lineno->udv_value, 0);
-    gpval_lineno->udv_undef = FALSE;
 
     lf_push(fp, name, NULL); /* save state for errors and recursion */
 
-    if (fp == (FILE *) NULL) {
-	int_error(NO_CARET, "Cannot open script file '%s'", name);
-	return; /* won't actually reach here */
-    }
-
     if (fp == stdin) {
 	/* DBT 10-6-98  go interactive if "-" named as load file */
 	interactive = TRUE;
@@ -328,14 +301,31 @@ load_file(FILE *fp, char *name, int calltype)
 
 	/* read one logical line */
 	while (more) {
-	    if (fgets(&(gp_input_line[start]), left, fp) == (char *) NULL) {
-		stop = TRUE;	/* EOF in file */
+	    if (fp && fgets(&(gp_input_line[start]), left, fp) == (char *) NULL) {
+		/* EOF in input file */
+		stop = TRUE;
+		gp_input_line[start] = '\0';
+		more = FALSE;
+	    } else if (!fp && datablock_input_line && (*datablock_input_line == NULL)) {
+		/* End of input datablock */
+		stop = TRUE;
 		gp_input_line[start] = '\0';
 		more = FALSE;
 	    } else {
+		/* Either we successfully read a line from input file fp
+		 * or we are about to copy a line from a datablock.
+		 * Either way we have to process line-ending '\' as a 
+		 * continuation request.
+		 */
+		if (!fp && datablock_input_line) {
+		    strncpy(&(gp_input_line[start]), *datablock_input_line, left);
+		    datablock_input_line++;
+		}
 		inline_num++;
 		gpval_lineno->udv_value.v.int_val = inline_num;	/* User visible copy */
-		len = strlen(gp_input_line) - 1;
+		if ((len = strlen(gp_input_line)) == 0)
+		    continue;
+		--len;
 		if (gp_input_line[len] == '\n') {	/* remove any newline */
 		    gp_input_line[len] = '\0';
 		    /* Look, len was 1-1 = 0 before, take care here! */
@@ -387,13 +377,17 @@ load_file(FILE *fp, char *name, int calltype)
 			left = gp_input_line_len - start;
 			continue;
 		    }
-		    
+
 		    more = FALSE;
 		}
 
 	    }
 	}
 
+	/* If we hit a 'break' or 'continue' statement in the lines just processed */
+	if (iteration_early_exit())
+	    continue;
+
 	/* process line */
 	if (strlen(gp_input_line) > 0) {
 #ifdef OLD_STYLE_CALL_ARGS
@@ -447,19 +441,34 @@ lf_pop()
 	if ((udv = get_udv_by_name("ARGC"))) {
 	    Ginteger(&(udv->udv_value), call_argc);
 	}
+
 	if ((udv = get_udv_by_name("ARG0"))) {
 	    gpfree_string(&(udv->udv_value));
 	    Gstring(&(udv->udv_value),
 		(lf->prev && lf->prev->name) ? gp_strdup(lf->prev->name) : gp_strdup(""));
 	}
+
 	for (argindex = 1; argindex <= 9; argindex++) {
 	    if ((udv = get_udv_by_name(argname[argindex]))) {
 		gpfree_string(&(udv->udv_value));
-		Gstring(&(udv->udv_value), gp_strdup(call_args[argindex-1]));
 		if (!call_args[argindex-1])
-		    udv->udv_undef = TRUE;
+		    udv->udv_value.type = NOTDEFINED;
+		else
+		    Gstring(&(udv->udv_value), gp_strdup(call_args[argindex-1]));
 	    }
 	}
+
+	if ((udv = get_udv_by_name("ARGV")) && udv->udv_value.type == ARRAY) {
+	    struct value *ARGV;
+	    int argv_size = lf->argv[0].v.int_val;
+
+	    gpfree_array(&(udv->udv_value));
+	    udv->udv_value.type = ARRAY;
+	    ARGV = udv->udv_value.v.value_array = gp_alloc((argv_size + 1) * sizeof(t_value), "array state");
+	    for (argindex = 0; argindex <= argv_size; argindex++)
+		ARGV[argindex] = lf->argv[argindex];
+	}
+
     }
 
     interactive = lf->interactive;
@@ -484,7 +493,7 @@ lf_pop()
     }
     free(lf->name);
     free(lf->cmdline);
-    
+
     lf_head = lf->prev;
     free(lf);
     return (TRUE);
@@ -519,10 +528,23 @@ lf_push(FILE *fp, char *name, char *cmdline)
 
     /* Call arguments are irrelevant if invoked from do_string_and_free */
     if (cmdline == NULL) {
+	struct udvt_entry *udv;
+	/* Save ARG0 through ARG9 */
 	for (argindex = 0; argindex < 10; argindex++) {
 	    lf->call_args[argindex] = call_args[argindex];
 	    call_args[argindex] = NULL;	/* initially no args */
 	}
+	/* Save ARGV[] */
+	lf->argv[0].v.int_val = 0;
+	lf->argv[0].type = NOTDEFINED;
+	if ((udv = get_udv_by_name("ARGV")) && udv->udv_value.type == ARRAY) {
+	    for (argindex = 0; argindex <= call_argc; argindex++) {
+		lf->argv[argindex] = udv->udv_value.v.value_array[argindex];
+		if (lf->argv[argindex].type == STRING)
+		    lf->argv[argindex].v.string_val =
+			gp_strdup(lf->argv[argindex].v.string_val);
+	    }
+	}
     }
     lf->depth = lf_head ? lf_head->depth+1 : 0;	/* recursion depth */
     if (lf->depth > STACK_DEPTH)
@@ -564,6 +586,15 @@ loadpath_fopen(const char *filename, const char *mode)
 {
     FILE *fp;
 
+    /* The global copy of fullname is only for the benefit of post.trm's
+     * automatic fontfile conversion via a constructed shell command.
+     * FIXME: There was a Feature Request to export the directory path
+     * in which a loaded file was found to a user-visible variable for the
+     * lifetime of that load.  This is close but without the lifetime.
+     */
+    free(loadpath_fontname);
+    loadpath_fontname = NULL;
+
 #if defined(PIPES)
     if (*filename == '<') {
 	restrict_popen();
@@ -581,7 +612,8 @@ loadpath_fopen(const char *filename, const char *mode)
 	    strcpy(fullname, path);
 	    PATH_CONCAT(fullname, filename);
 	    if ((fp = fopen(fullname, mode)) != NULL) {
-		free(fullname);
+		/* free(fullname); */
+		loadpath_fontname = fullname;
 		fullname = NULL;
 		/* reset loadpath internals!
 		 * maybe this can be replaced by calling get_loadpath with
@@ -591,150 +623,16 @@ loadpath_fopen(const char *filename, const char *mode)
 	    }
 	}
 
-	if (fullname)
-	    free(fullname);
-
+	free(fullname);
     }
 
-#ifdef _Windows
+#ifdef _WIN32
     if (fp != NULL)
-	setmode(fileno(fp), _O_BINARY);
+	_setmode(_fileno(fp), _O_BINARY);
 #endif
     return fp;
 }
 
-
-/* Harald Harders <h.harders@tu-bs.de> */
-/* Thanks to John Bollinger <jab@bollingerbands.com> who has tested the
-   windows part */
-static char *
-recursivefullname(const char *path, const char *filename, TBOOLEAN recursive)
-{
-    char *fullname = NULL;
-    FILE *fp;
-
-    /* length of path, dir separator, filename, \0 */
-    fullname = gp_alloc(strlen(path) + 1 + strlen(filename) + 1,
-			"recursivefullname");
-    strcpy(fullname, path);
-    PATH_CONCAT(fullname, filename);
-
-    if ((fp = fopen(fullname, "r")) != NULL) {
-	fclose(fp);
-	return fullname;
-    } else {
-	free(fullname);
-	fullname = NULL;
-    }
-
-    if (recursive) {
-#ifdef HAVE_DIRENT_H
-	DIR *dir;
-	struct dirent *direntry;
-	struct stat buf;
-
-	dir = opendir(path);
-	if (dir) {
-	    while ((direntry = readdir(dir)) != NULL) {
-		char *fulldir = gp_alloc(strlen(path) + 1 + strlen(direntry->d_name) + 1,
-					 "fontpath_fullname");
-		strcpy(fulldir, path);
-#  if defined(VMS)
-		if (fulldir[strlen(fulldir) - 1] == ']')
-		    fulldir[strlen(fulldir) - 1] = '\0';
-		strcpy(&(fulldir[strlen(fulldir)]), ".");
-		strcpy(&(fulldir[strlen(fulldir)]), direntry->d_name);
-		strcpy(&(fulldir[strlen(fulldir)]), "]");
-#  else
-		PATH_CONCAT(fulldir, direntry->d_name);
-#  endif
-		stat(fulldir, &buf);
-		if ((S_ISDIR(buf.st_mode)) &&
-		    (strcmp(direntry->d_name, ".") != 0) &&
-		    (strcmp(direntry->d_name, "..") != 0)) {
-		    fullname = recursivefullname(fulldir, filename, TRUE);
-		    if (fullname != NULL)
-			break;
-		}
-		free(fulldir);
-	    }
-	    closedir(dir);
-	}
-#elif defined(_Windows)
-	HANDLE filehandle;
-	WIN32_FIND_DATA finddata;
-	char *pathwildcard = gp_alloc(strlen(path) + 2, "fontpath_fullname");
-
-	strcpy(pathwildcard, path);
-	PATH_CONCAT(pathwildcard, "*");
-
-	filehandle = FindFirstFile(pathwildcard, &finddata);
-	free(pathwildcard);
-	if (filehandle != INVALID_HANDLE_VALUE)
-	    do {
-		if ((finddata.dwFileAttributes & FILE_ATTRIBUTE_DIRECTORY) &&
-		    (strcmp(finddata.cFileName, ".") != 0) &&
-		    (strcmp(finddata.cFileName, "..") != 0)) {
-		    char *fulldir = gp_alloc(strlen(path) + 1 + strlen(finddata.cFileName) + 1,
-					     "fontpath_fullname");
-		    strcpy(fulldir, path);
-		    PATH_CONCAT(fulldir, finddata.cFileName);
-
-		    fullname = recursivefullname(fulldir, filename, TRUE);
-		    free(fulldir);
-		    if (fullname != NULL)
-			break;
-		}
-	    } while (FindNextFile(filehandle, &finddata) != 0);
-	FindClose(filehandle);
-
-#else
-	int_warn(NO_CARET, "Recursive directory search not supported\n\t('%s!')", path);
-#endif
-    }
-    return fullname;
-}
-
-
-/* may return NULL */
-char *
-fontpath_fullname(const char *filename)
-{
-    FILE *fp;
-    char *fullname = NULL;
-
-#if defined(PIPES)
-    if (*filename == '<') {
-	os_error(NO_CARET, "fontpath_fullname: No Pipe allowed");
-    } else
-#endif /* PIPES */
-    if ((fp = fopen(filename, "r")) == (FILE *) NULL) {
-	/* try 'fontpath' variable */
-	char *tmppath, *path = NULL;
-
-	while ((tmppath = get_fontpath()) != NULL) {
-	    TBOOLEAN subdirs = FALSE;
-	    path = gp_strdup(tmppath);
-	    if (path[strlen(path) - 1] == '!') {
-		path[strlen(path) - 1] = '\0';
-		subdirs = TRUE;
-	    }			/* if */
-	    fullname = recursivefullname(path, filename, subdirs);
-	    if (fullname != NULL) {
-		while (get_fontpath());
-		free(path);
-		break;
-	    }
-	    free(path);
-	}
-
-    } else
-	fullname = gp_strdup(filename);
-
-    return fullname;
-}
-
-
 /* Push current terminal.
  * Called 1. in main(), just after init_terminal(),
  *        2. from load_rcfile(),
@@ -807,47 +705,43 @@ get_style()
 }
 
 /* Parse options for style filledcurves and fill fco accordingly.
- * If no option given, then set fco->opt_given to 0.
+ * If no option given, set it to FILLEDCURVES_DEFAULT.
  */
 void
 get_filledcurves_style_options(filledcurves_opts *fco)
 {
-    int p;
-    p = lookup_table(&filledcurves_opts_tbl[0], c_token);
+    enum filledcurves_opts_id p;
+    fco->closeto = FILLEDCURVES_DEFAULT;
+    fco->oneside = 0;
 
-    if (p == FILLEDCURVES_ABOVE) {
-	fco->oneside = 1;
-	p = lookup_table(&filledcurves_opts_tbl[0], ++c_token);
-    } else if (p == FILLEDCURVES_BELOW) {
-	fco->oneside = -1;
-	p = lookup_table(&filledcurves_opts_tbl[0], ++c_token);
-    } else
-	fco->oneside = 0;
+    while ((p = lookup_table(&filledcurves_opts_tbl[0], c_token)) != -1) {
+	fco->closeto = p;
+	c_token++;
 
-    if (p == -1) {
-	fco->opt_given = 0;
-	return;			/* no option given */
-    } else
-	fco->opt_given = 1;
+	if (p == FILLEDCURVES_ABOVE) {
+	    fco->oneside = 1;
+	    continue;
+	} else if (p == FILLEDCURVES_BELOW) {
+	    fco->oneside = -1;
+	    continue;
+	}
 
-    c_token++;
-
-    fco->closeto = p;
-    fco->at = 0;
-    if (!equals(c_token, "="))
-	return;
-    /* parameter required for filledcurves x1=... and friends */
-    if (p < FILLEDCURVES_ATXY)
-	fco->closeto += 4;
-    c_token++;
-    fco->at = real_expression();
-    if (p != FILLEDCURVES_ATXY)
-	return;
-    /* two values required for FILLEDCURVES_ATXY */
-    if (!equals(c_token, ","))
-	int_error(c_token, "syntax is xy=<x>,<y>");
-    c_token++;
-    fco->aty = real_expression();
+	fco->at = 0;
+	if (!equals(c_token, "="))
+	    return;
+	/* parameter required for filledcurves x1=... and friends */
+	if (p < FILLEDCURVES_ATXY)
+	    fco->closeto += 4;
+	c_token++;
+	fco->at = real_expression();
+	if (p != FILLEDCURVES_ATXY)
+	    return;
+	/* two values required for FILLEDCURVES_ATXY */
+	if (!equals(c_token, ","))
+	    int_error(c_token, "syntax is xy=<x>,<y>");
+	c_token++;
+	fco->aty = real_expression();
+    }
     return;
 }
 
@@ -857,7 +751,7 @@ get_filledcurves_style_options(filledcurves_opts *fco)
 void
 filledcurves_options_tofile(filledcurves_opts *fco, FILE *fp)
 {
-    if (!fco->opt_given)
+    if (fco->closeto == FILLEDCURVES_DEFAULT)
 	return;
     if (fco->oneside)
 	fputs(fco->oneside > 0 ? "above " : "below ", fp);
@@ -894,8 +788,8 @@ need_fill_border(struct fill_style_type *fillstyle)
 
     /* Wants a border in a new color */
     if (p.pm3d_color.type != TC_DEFAULT)
-	apply_pm3dcolor(&p.pm3d_color,term);
-    
+	apply_pm3dcolor(&p.pm3d_color);
+
     return TRUE;
 }
 
@@ -910,7 +804,7 @@ parse_dashtype(struct t_dashtype *dt)
     /* Erase any previous contents */
     memset(dt, 0, sizeof(struct t_dashtype));
 
-    /* Fill in structure based on keyword ... */ 
+    /* Fill in structure based on keyword ... */
     if (equals(c_token, "solid")) {
 	res = DASHTYPE_SOLID;
 	c_token++;
@@ -942,7 +836,7 @@ parse_dashtype(struct t_dashtype *dt)
     } else if ((dash_str = try_to_get_string())) {
 #define DSCALE 10.
 	while (dash_str[j] && (k < DASHPATTERN_LENGTH || dash_str[j] == ' ')) {
-	    /* .      Dot with short space 
+	    /* .      Dot with short space
 	     * -      Dash with regular space
 	     * _      Long dash with regular space
 	     * space  Don't add new dash, just increase last space */
@@ -971,17 +865,19 @@ parse_dashtype(struct t_dashtype *dt)
 	}
 	/* truncate dash_str if we ran out of space in the array representation */
 	dash_str[j] = '\0';
-	strncpy(dt->dstring, dash_str, sizeof(dt->dstring)-1);
+	safe_strncpy(dt->dstring, dash_str, sizeof(dt->dstring));
 	free(dash_str);
 	res = DASHTYPE_CUSTOM;
 
     /* Or index of previously defined dashtype */
-    /* FIXME: Is the index enough or should we copy its contents into this one? */
-    /* FIXME: What happens if there is a recursive definition? */
     } else {
-	res = int_expression() - 1;
+	res = int_expression();
 	if (res < 0)
-	    int_error(c_token - 1, "tag must be > 0");
+	    int_error(c_token - 1, "dashtype must be non-negative");
+	if (res == 0)
+	    res = DASHTYPE_AXIS;
+	else
+	    res = res - 1;
     }
 
     return res;
@@ -989,7 +885,7 @@ parse_dashtype(struct t_dashtype *dt)
 
 /*
  * destination_class tells us whether we are filling in a line style ('set style line'),
- * a persistant linetype ('set linetype') or an ad hoc set of properties for a single
+ * a persistent linetype ('set linetype') or an ad hoc set of properties for a single
  * use ('plot ... lc foo lw baz').
  * allow_point controls whether we accept a point attribute in this lp_style.
  */
@@ -997,27 +893,28 @@ int
 lp_parse(struct lp_style_type *lp, lp_class destination_class, TBOOLEAN allow_point)
 {
     /* keep track of which options were set during this call */
-    int set_lt = 0, set_pal = 0, set_lw = 0; 
+    int set_lt = 0, set_pal = 0, set_lw = 0;
     int set_pt = 0, set_ps  = 0, set_pi = 0;
+    int set_pn = 0;
     int set_dt = 0;
     int new_lt = 0;
 
     /* EAM Mar 2010 - We don't want properties from a user-defined default
      * linetype to override properties explicitly set here.  So fill in a
      * local lp_style_type as we go and then copy over the specifically
-     * requested properties on top of the default ones.                                           
+     * requested properties on top of the default ones.
      */
     struct lp_style_type newlp = *lp;
-	
+
     if ((destination_class == LP_ADHOC)
     && (almost_equals(c_token, "lines$tyle") || equals(c_token, "ls"))) {
 	c_token++;
 	lp_use_properties(lp, int_expression());
-    } 
-    
+    }
+
     while (!END_OF_COMMAND) {
 
-	/* This special case is to flag an attemp to "set object N lt <lt>",
+	/* This special case is to flag an attempt to "set object N lt <lt>",
 	 * which would otherwise be accepted but ignored, leading to confusion
 	 * FIXME:  Couldn't this be handled at a higher level?
 	 */
@@ -1074,11 +971,15 @@ lp_parse(struct lp_style_type *lp, lp_class destination_class, TBOOLEAN allow_po
 	    continue;
 	}
 
-	/* This is so that "set obj ... lw N fc <colorspec>" doesn't eat */
-	/* up the colorspec as a line property.  We need to parse it later */
-	/* as a _fill_ property */
-	if ((destination_class == LP_NOFILL)
-	&&  (equals(c_token,"fc") || almost_equals(c_token,"fillc$olor")))
+	/* This is so that "set obj ... lw N fc <colorspec>" doesn't eat up the
+	 * fc colorspec as a line property.  We need to parse it later as a
+	 * _fill_ property. Also prevents "plot ... fc <col1> fs <foo> lw <baz>"
+	 * from generating an error claiming redundant line properties.
+	 */
+	if ((destination_class == LP_NOFILL || destination_class == LP_ADHOC)
+	&&  (equals(c_token,"fc") || almost_equals(c_token,"fillc$olor"))
+	&&  (!almost_equals(c_token+1, "pal$ette"))
+	   )
 	    break;
 
 	if (equals(c_token,"lc") || almost_equals(c_token,"linec$olor")
@@ -1154,16 +1055,12 @@ lp_parse(struct lp_style_type *lp, lp_class destination_class, TBOOLEAN allow_po
 		c_token++;
 		if ((symbol = try_to_get_string())) {
 		    newlp.p_type = PT_CHARACTER;
-		    /* An alternative mechanism would be to use
-		     * utf8toulong(&newlp.p_char, symbol);
-		     */
-		    strncpy((char *)(&newlp.p_char), symbol, 3);
-		    /* Truncate ascii text to single character */
-		    if ((((char *)&newlp.p_char)[0] & 0x80) == 0)
-			((char *)&newlp.p_char)[1] = '\0';
-		    /* UTF-8 characters may use up to 3 bytes */
-		    ((char *)&newlp.p_char)[3] = '\0';
+		    truncate_to_one_utf8_char(symbol);
+		    safe_strncpy(newlp.p_char, symbol, sizeof(newlp.p_char));
 		    free(symbol);
+		} else if (almost_equals(c_token, "var$iable") && (destination_class == LP_ADHOC)) {
+		    newlp.p_type = PT_VARIABLE;
+		    c_token++;
 		} else {
 		    newlp.p_type = int_expression() - 1;
 		}
@@ -1203,7 +1100,19 @@ lp_parse(struct lp_style_type *lp, lp_class destination_class, TBOOLEAN allow_po
 		newlp.p_interval = int_expression();
 		set_pi = 1;
 	    } else {
-		int_warn(c_token, "No pointinterval specifier allowed, here");
+		int_warn(c_token, "No pointinterval specifier allowed here");
+		int_expression();
+	    }
+	    continue;
+	}
+
+	if (almost_equals(c_token, "pointn$umber") || equals(c_token, "pn")) {
+	    c_token++;
+	    if (allow_point) {
+		newlp.p_number = int_expression();
+		set_pn = 1;
+	    } else {
+		int_warn(c_token, "No pointnumber specifier allowed here)");
 		int_expression();
 	    }
 	    continue;
@@ -1216,7 +1125,9 @@ lp_parse(struct lp_style_type *lp, lp_class destination_class, TBOOLEAN allow_po
 	    c_token++;
 	    tmp = parse_dashtype(&newlp.custom_dash_pattern);
 	    /* Pull the dashtype from the list of already defined dashtypes, */
-	    /* but only if it we didn't get an explicit one back from parse_dashtype */ 
+	    /* but only if it we didn't get an explicit one back from parse_dashtype */
+	    if (tmp == DASHTYPE_AXIS)
+		lp->l_type = LT_AXIS;
 	    if (tmp >= 0)
 		tmp = load_dashtype(&newlp.custom_dash_pattern, tmp + 1);
 	    newlp.d_type = tmp;
@@ -1228,8 +1139,9 @@ lp_parse(struct lp_style_type *lp, lp_class destination_class, TBOOLEAN allow_po
 	break;
     }
 
-    if (set_lt > 1 || set_pal > 1 || set_lw > 1 || set_pt > 1 || set_ps > 1 || set_dt > 1)
-	int_error(c_token, "duplicated arguments in style specification");
+    if (set_lt > 1 || set_pal > 1 || set_lw > 1 || set_pt > 1 || set_ps > 1 || set_dt > 1
+    || (set_pi + set_pn > 1))
+	int_error(c_token, "duplicate or conflicting arguments in style specification");
 
     if (set_pal) {
 	lp->pm3d_color = newlp.pm3d_color;
@@ -1242,38 +1154,44 @@ lp_parse(struct lp_style_type *lp, lp_class destination_class, TBOOLEAN allow_po
 	lp->l_width = newlp.l_width;
     if (set_pt) {
 	lp->p_type = newlp.p_type;
-	lp->p_char = newlp.p_char;
+	memcpy(lp->p_char, newlp.p_char, sizeof(newlp.p_char));
     }
     if (set_ps)
 	lp->p_size = newlp.p_size;
-    if (set_pi)
+    if (set_pi) {
 	lp->p_interval = newlp.p_interval;
+	lp->p_number = 0;
+    }
+    if (set_pn) {
+	lp->p_number = newlp.p_number;
+	lp->p_interval = 0;
+    }
     if (newlp.l_type == LT_COLORFROMCOLUMN)
 	lp->l_type = LT_COLORFROMCOLUMN;
     if (set_dt) {
 	lp->d_type = newlp.d_type;
 	lp->custom_dash_pattern = newlp.custom_dash_pattern;
-    }	
-		
+    }
+
 
     return new_lt;
 }
 
 /* <fillstyle> = {empty | solid {<density>} | pattern {<n>}} {noborder | border {<lt>}} */
+const struct gen_table fs_opt_tbl[] = {
+    {"e$mpty", FS_EMPTY},
+    {"s$olid", FS_SOLID},
+    {"p$attern", FS_PATTERN},
+    {NULL, -1}
+};
+
 void
-parse_fillstyle(struct fill_style_type *fs, int def_style, int def_density, int def_pattern, 
-		t_colorspec def_bordertype)
+parse_fillstyle(struct fill_style_type *fs)
 {
     TBOOLEAN set_fill = FALSE;
-    TBOOLEAN set_param = FALSE;
+    TBOOLEAN set_border = FALSE;
     TBOOLEAN transparent = FALSE;
 
-    /* Set defaults */
-    fs->fillstyle = def_style;
-    fs->filldensity = def_density;
-    fs->fillpattern = def_pattern;
-    fs->border_color = def_bordertype;
-
     if (END_OF_COMMAND)
 	return;
     if (!equals(c_token, "fs") && !almost_equals(c_token, "fill$style"))
@@ -1281,29 +1199,59 @@ parse_fillstyle(struct fill_style_type *fs, int def_style, int def_density, int
     c_token++;
 
     while (!END_OF_COMMAND) {
+	int i;
+
 	if (almost_equals(c_token, "trans$parent")) {
 	    transparent = TRUE;
+	    fs->filldensity = 50;
 	    c_token++;
-	}
-
-	if (almost_equals(c_token, "e$mpty")) {
-	    fs->fillstyle = FS_EMPTY;
-	    c_token++;
-	} else if (almost_equals(c_token, "s$olid")) {
-	    fs->fillstyle = FS_SOLID;
-	    set_fill = TRUE;
-	    c_token++;
-	} else if (almost_equals(c_token, "p$attern")) {
-	    fs->fillstyle = FS_PATTERN;
-	    set_fill = TRUE;
-	    c_token++;
-	}
-
-	if (END_OF_COMMAND)
 	    continue;
-	else if (almost_equals(c_token, "bo$rder")) {
+	}
+
+	i = lookup_table(fs_opt_tbl, c_token);
+	switch (i) {
+	    default:
+		 break;
+
+	    case FS_EMPTY:
+	    case FS_SOLID:
+	    case FS_PATTERN:
+
+		if (set_fill && fs->fillstyle != i)
+		    int_error(c_token, "conflicting option");
+		fs->fillstyle = i;
+		set_fill = TRUE;
+		c_token++;
+
+		if (!transparent)
+			fs->filldensity = 100;
+
+		if (might_be_numeric(c_token)) {
+		    if (fs->fillstyle == FS_SOLID) {
+			/* user sets 0...1, but is stored as an integer 0..100 */
+			fs->filldensity = 100.0 * real_expression() + 0.5;
+			if (fs->filldensity < 0)
+			    fs->filldensity = 0;
+			if (fs->filldensity > 100)
+			    fs->filldensity = 100;
+		    } else if (fs->fillstyle == FS_PATTERN) {
+			fs->fillpattern = int_expression();
+			if (fs->fillpattern < 0)
+			    fs->fillpattern = 0;
+		    } else
+			int_error(c_token, "this fill style does not have a parameter");
+		}
+		continue;
+	}
+
+	if (almost_equals(c_token, "bo$rder")) {
+	    if (set_border && fs->border_color.lt == LT_NODRAW)
+		int_error(c_token, "conflicting option");
 	    fs->border_color.type = TC_DEFAULT;
+	    set_border = TRUE;
 	    c_token++;
+	    if (END_OF_COMMAND)
+		continue;
 	    if (equals(c_token,"-") || isanumber(c_token)) {
 		fs->border_color.type = TC_LT;
 		fs->border_color.lt = int_expression() - 1;
@@ -1316,37 +1264,22 @@ parse_fillstyle(struct fill_style_type *fs, int def_style, int def_density, int
 	    }
 	    continue;
 	} else if (almost_equals(c_token, "nobo$rder")) {
+	    if (set_border && fs->border_color.lt != LT_NODRAW)
+		int_error(c_token, "conflicting option");
 	    fs->border_color.type = TC_LT;
 	    fs->border_color.lt = LT_NODRAW;
+	    set_border = TRUE;
 	    c_token++;
 	    continue;
 	}
 
-	/* We hit something unexpected */
-	if (!set_fill || set_param)
-	    break;
-	if (!(isanumber(c_token) || type_udv(c_token) == INTGR || type_udv(c_token) == CMPLX))
-	    break;
-
-	if (fs->fillstyle == FS_SOLID) {
-	    /* user sets 0...1, but is stored as an integer 0..100 */
-	    fs->filldensity = 100.0 * real_expression() + 0.5;
-	    if (fs->filldensity < 0)
-		fs->filldensity = 0;
-	    if (fs->filldensity > 100)
-		fs->filldensity = 100;
-	    set_param = TRUE;
-	} else if (fs->fillstyle == FS_PATTERN) {
-	    fs->fillpattern = int_expression();
-	    if (fs->fillpattern < 0)
-		fs->fillpattern = 0;
-	    set_param = TRUE;
-	}
+	/* Keyword must belong to someone else */
+	break;
     }
     if (transparent) {
 	if (fs->fillstyle == FS_SOLID)
 	    fs->fillstyle = FS_TRANSPARENT_SOLID;
-        else if (fs->fillstyle == FS_PATTERN)
+	else if (fs->fillstyle == FS_PATTERN)
 	    fs->fillstyle = FS_TRANSPARENT_PATTERN;
     }
 }
@@ -1392,8 +1325,6 @@ parse_colorspec(struct t_colorspec *tc, int options)
 	/*
 	 * July 2014 - translate linetype into user-defined linetype color.
 	 * This is a CHANGE!
-	 * FIXME: calling load_linetype here may obviate the need to call it
-	 * many places in the higher level code.  They could be removed.
 	 */
 	load_linetype(&lptemp, tc->lt + 1);
 	*tc = lptemp.pm3d_color;
@@ -1499,19 +1430,29 @@ arrow_use_properties(struct arrow_style_type *arrow, int tag)
      *  copies its data into the structure 'ap'. */
     struct arrowstyle_def *this;
 
+    /* If a color has already been set for this arrow, keep it */
+    struct t_colorspec save_colorspec = arrow->lp_properties.pm3d_color;
+
+    /* Default if requested style is not found */
+    default_arrow_style(arrow);
+
     this = first_arrowstyle;
     while (this != NULL) {
 	if (this->tag == tag) {
 	    *arrow = this->arrow_properties;
-	    return;
+	    break;
 	} else {
 	    this = this->next;
 	}
     }
 
     /* tag not found: */
-    default_arrow_style(arrow);
-    int_warn(NO_CARET,"arrowstyle %d not found", tag);
+    if (!this || this->tag != tag)
+	int_warn(NO_CARET,"arrowstyle %d not found", tag);
+
+    /* Restore original color if the style doesn't specify one */
+    if (arrow->lp_properties.pm3d_color.type == TC_DEFAULT)
+	arrow->lp_properties.pm3d_color = save_colorspec;
 }
 
 void
@@ -1613,7 +1554,7 @@ arrow_parse(
 	    /* invalid backangle --> default of 90.0 degrees */
 	    if (arrow->head_backangle <= arrow->head_angle)
 		arrow->head_backangle = 90.0;
-	    
+
 	    /* Assume adjustable size but check for 'fixed' instead */
 	    arrow->head_fixedsize = FALSE;
 	    continue;
