@@ -55,6 +55,10 @@ typedef enum t_fillstyle { FS_EMPTY, FS_SOLID, FS_PATTERN, FS_DEFAULT, FS_TRANSP
 #include <QtGui>
 #include <QDebug>
 
+#if QT_VERSION < QT_VERSION_CHECK(5, 11, 0)
+# define horizontalAdvance width
+#endif
+
 QtGnuplotScene::QtGnuplotScene(QtGnuplotEventHandler* eventHandler, QObject* parent)
 	: QGraphicsScene(parent)
 {
@@ -163,7 +167,7 @@ void QtGnuplotScene::flushCurrentPointsItem()
 void QtGnuplotScene::update_key_box(const QRectF rect)
 {
 	if (m_currentPlotNumber > m_key_boxes.count()) {
-		m_key_boxes.insert(m_currentPlotNumber, QtGnuplotKeybox(rect));
+		m_key_boxes.insert(m_currentPlotNumber-1, QtGnuplotKeybox(rect));
 	} else if (m_key_boxes[m_currentPlotNumber-1].isEmpty()) {
 		// Retain the visible/hidden flag when re-initializing the Keybox
 		bool tmp = m_key_boxes[m_currentPlotNumber-1].ishidden();
@@ -186,7 +190,6 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 	if (type == GEClear)
 	{
 		resetItems();
-		m_preserve_visibility = false;
 	}
 	else if (type == GELineWidth)
 	{
@@ -266,7 +269,14 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 		QPolygonF polygon; in >> polygon;
 
 		if (!m_inKeySample)
+		{
+			// Distinguish between opaque and transparent pattern fill
+			if (m_currentFillStyle ==  FS_PATTERN)
+				m_currentPointsItem->addFilledPolygon(clipPolygon(polygon, false), 
+					QBrush(m_widget->backgroundColor()));
+
 			m_currentPointsItem->addFilledPolygon(clipPolygon(polygon, false), m_currentBrush);
+		}
 		else
 		{
 			flushCurrentPointsItem();
@@ -293,8 +303,24 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 	{
 		QString fontName; in >> fontName;
 		int size        ; in >> size;
+
+		// strip :Bold or :Italic property out of font name
+		if (fontName.contains(":italic", Qt::CaseInsensitive))
+			m_font.setStyle(QFont::StyleItalic);
+		else
+			m_font.setStyle(QFont::StyleNormal);
+		if (fontName.contains(":bold", Qt::CaseInsensitive))
+			m_font.setWeight(QFont::Bold);
+		else
+			m_font.setWeight(QFont::Normal);
+		int sep = fontName.indexOf(":");
+		if (sep >= 0)
+			fontName.truncate(sep);
+
+		// Blank font name means keep using the previous font
 		m_font.setFamily(fontName);
 		m_font.setPointSize(size);
+		m_font.setStyleStrategy(QFont::ForceOutline);	// pcf fonts die if rotated
 	}
 	else if (type == GEPoint)
 	{
@@ -320,7 +346,7 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 		// Create a hypertext label that will become visible on mouseover.
 		// The Z offset is a kludge to force the label into the foreground.
 		if (!m_currentHypertext.isEmpty()) {
-			QGraphicsTextItem* textItem = addText(m_currentHypertext, m_font);
+			QGraphicsTextItem* textItem = addText(m_currentHypertext, m_hypertextFont);
 			textItem->setPos(point + m_textOffset);
 			textItem->setZValue(m_currentZ+10000);
 			textItem->setVisible(false);
@@ -338,10 +364,10 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 		positionText(textItem, point);
 
 		QRectF rect = textItem->boundingRect();
-		if (m_textAlignment == Qt::AlignCenter) {
+		if (m_textAlignment & Qt::AlignCenter) {
 			rect.moveCenter(point);
 			rect.moveBottom(point.y());
-		} else if (m_textAlignment == Qt::AlignRight)
+		} else if (m_textAlignment & Qt::AlignRight)
 			rect.moveBottomRight(point);
 		else
 			rect.moveBottomLeft(point);
@@ -351,10 +377,12 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 			update_key_box(rect);
 		else
 			m_currentGroup.append(textItem);
-#ifdef EAM_BOXED_TEXT
-		if (m_inTextBox)
+
+		if (m_inTextBox) {
 			m_currentTextBox |= rect;
-#endif
+			m_currentBoxRotation = m_textAngle;
+			m_currentBoxOrigin = point;
+		}
 	}
 	else if (type == GEEnhancedFlush)
 	{
@@ -384,10 +412,10 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 		addItem(m_enhanced);
 
 		QRectF rect = m_enhanced->boundingRect();
-		if (m_textAlignment == Qt::AlignCenter) {
+		if (m_textAlignment & Qt::AlignCenter) {
 			rect.moveCenter(point);
 			rect.moveBottom(point.y());
-		} else if (m_textAlignment == Qt::AlignRight)
+		} else if (m_textAlignment & Qt::AlignRight)
 			rect.moveBottomRight(point);
 		else
 			rect.moveBottomLeft(point);
@@ -398,10 +426,12 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 			update_key_box(rect);
 		else
 			m_currentGroup.append(m_enhanced);
-#ifdef EAM_BOXED_TEXT
-		if (m_inTextBox)
+
+		if (m_inTextBox) {
 			m_currentTextBox |= rect;
-#endif
+			m_currentBoxRotation = m_textAngle;
+			m_currentBoxOrigin = point;
+		}
 		m_enhanced = 0;
 	}
 	else if (type == GEImage)
@@ -447,7 +477,6 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 			m_zoomStopText->setPlainText(text); /// @todo font
 			m_zoomStopText->setPos(m_lastMousePos);
 			m_zoomRect->setRect(QRectF(m_zoomBoxCorner + QPointF(0.5, 0.5), m_lastMousePos + QPointF(0.5, 0.5)).normalized());
-			m_zoomRect->setZValue(32767);  // make sure guide box is on top
 		}
 	}
 	else if (type == GELineTo)
@@ -470,6 +499,7 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 	{
 		for (int i = 0; i < 4; i++)
 			in >> m_axisValid[i] >> m_axisMin[i] >> m_axisLower[i] >> m_axisScale[i] >> m_axisLog[i];
+		in >> m_axisValid[4];
 	}
 	else if (type == GEAfterPlot) 
 	{
@@ -483,8 +513,15 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 			if (0 < m_currentPlotNumber && m_currentPlotNumber <= m_key_boxes.count())
 				newgroup->setVisible( !(m_key_boxes[m_currentPlotNumber-1].ishidden()) );
 			// Store it in an ordered list so we can toggle it by index
-			m_plot_group.insert(m_currentPlotNumber, newgroup);
+			m_plot_group.insert(m_currentPlotNumber-1, newgroup);
 		} 
+
+		if (m_currentPlotNumber >= m_key_boxes.count()) {
+			QRectF empty( QPointF(0,0), QPointF(0,0));
+			m_key_boxes.append(empty);
+			m_key_boxes[m_currentPlotNumber-1].resetStatus();
+		}
+
 		m_currentPlotNumber = 0;
 	}
 	else if (type == GEPlotNumber) 
@@ -509,7 +546,6 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 		enum QtGnuplotModPlots ops = (enum QtGnuplotModPlots) ops_i;
 
 		/* FIXME: This shouldn't happen, but it does. */
-		/* Failure to reset lists after multiplot??   */
 		if (i > m_plot_group.count())
 		    i = m_plot_group.count();
 
@@ -553,7 +589,9 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 			// Draw an invisible grey rectangle in the key box.
 			// It will be set to visible if the plot is toggled off.
 			QtGnuplotKeybox *keybox = &m_key_boxes[m_currentPlotNumber-1];
-			QGraphicsRectItem *statusBox = addRect(*keybox, Qt::NoPen, Qt::Dense4Pattern);
+			m_currentBrush.setColor(Qt::lightGray);
+			m_currentBrush.setStyle(Qt::Dense4Pattern);
+			QGraphicsRectItem *statusBox = addRect(*keybox, Qt::NoPen, m_currentBrush);
 			statusBox->setZValue(m_currentZ-1);
 			keybox->showStatus(statusBox);
 		}
@@ -562,21 +600,24 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 	{
 		m_currentHypertext.clear();
 		in >> m_currentHypertext;
+		m_hypertextFont = m_font;
 	}
-#ifdef EAM_BOXED_TEXT
 	else if (type == GETextBox)
 	{
 		flushCurrentPointsItem();
 		QPointF point; in >> point;
 		int option; in >> option;
 		QGraphicsRectItem *rectItem;
+		QRectF outline;
 
 		/* Must match the definition in ../term_api.h */
 		enum t_textbox_options {
-		       TEXTBOX_INIT = 0,
-		       TEXTBOX_OUTLINE,
-		       TEXTBOX_BACKGROUNDFILL,
-		       TEXTBOX_MARGINS
+			TEXTBOX_INIT = 0,
+			TEXTBOX_OUTLINE,
+			TEXTBOX_BACKGROUNDFILL,
+			TEXTBOX_MARGINS,
+			TEXTBOX_FINISH,
+			TEXTBOX_GREY
 		};
 
 		switch (option) {
@@ -587,31 +628,42 @@ void QtGnuplotScene::processEvent(QtGnuplotEventType type, QDataStream& in)
 			break;
 		case TEXTBOX_OUTLINE:
 			/* Stroke bounding box */
-			rectItem = addRect(m_currentTextBox, m_currentPen, Qt::NoBrush);
+			outline = m_currentTextBox.adjusted(
+				 -m_textMargin.x(), -m_textMargin.y(),
+				  m_textMargin.x(), m_textMargin.y());
+			rectItem = addRect(outline, m_currentPen, Qt::NoBrush);
 			rectItem->setZValue(m_currentZ++);
+			rectItem->setTransformOriginPoint(m_currentBoxOrigin);
+			rectItem->setRotation(-m_currentBoxRotation);
 			m_currentGroup.append(rectItem);
 			m_inTextBox = false;
 			break;
 		case TEXTBOX_BACKGROUNDFILL:
 			/* Fill bounding box */
-			m_currentBrush.setColor(m_widget->backgroundColor());
+			m_currentBrush.setColor(m_currentPen.color());
 			m_currentBrush.setStyle(Qt::SolidPattern);
-			rectItem = addRect(m_currentTextBox, Qt::NoPen, m_currentBrush);
+			outline = m_currentTextBox.adjusted(
+				 -m_textMargin.x(), -m_textMargin.y(),
+				  m_textMargin.x(), m_textMargin.y());
+			rectItem = addRect(outline, Qt::NoPen, m_currentBrush);
 			rectItem->setZValue(m_currentZ++);
+			rectItem->setTransformOriginPoint(m_currentBoxOrigin);
+			rectItem->setRotation(-m_currentBoxRotation);
 			m_currentGroup.append(rectItem);
 			m_inTextBox = false;
 			break;
 		case TEXTBOX_MARGINS:
 			/* Set margins of bounding box */
+			m_textMargin = point;
+			m_textMargin *= QFontMetrics(m_font).averageCharWidth();
 			break;
 		}
 	}
-#endif
 	else if (type == GEFontMetricRequest)
 	{
 		QFontMetrics metrics(m_font);
 		int par1 = (metrics.ascent() + metrics.descent());
-		int par2 = metrics.width("0123456789")/10.;
+		int par2 = metrics.horizontalAdvance("0123456789")/10.;
 		m_eventHandler->postTermEvent(GE_fontprops, 0, 0, par1, par2, m_widget);
 	}
 	else if (type == GEDone)
@@ -633,10 +685,13 @@ void QtGnuplotScene::resetItems()
 
 	m_zoomRect = addRect(QRect(), QPen(QColor(0, 0, 0, 200)), QBrush(QColor(0, 0, 255, 40)));
 	m_zoomRect->setVisible(false);
+	m_zoomRect->setZValue(32767);       // make sure guide box is on top
 	m_zoomStartText = addText("");
 	m_zoomStopText  = addText("");
 	m_zoomStartText->setVisible(false);
 	m_zoomStopText->setVisible(false);
+	m_zoomStartText->setZValue(32767);  // make sure guide box annotation is on top
+	m_zoomStopText->setZValue(32767);
 	m_horizontalRuler = addLine(QLine(0, 0, width(), 0) , QPen(QColor(0, 0, 0, 200)));
 	m_verticalRuler   = addLine(QLine(0, 0, 0, height()), QPen(QColor(0, 0, 0, 200)));
 	m_lineTo          = addLine(QLine()                 , QPen(QColor(0, 0, 0, 200)));
@@ -656,6 +711,9 @@ void QtGnuplotScene::resetItems()
 	m_hypertextList.append(addRect(QRect(), QPen(QColor(0, 0, 0, 100)), QBrush(QColor(225, 225, 225, 200))));
 	m_hypertextList[0]->setVisible(false);
 
+	m_hyperimage = addPixmap(QPixmap());
+	m_hyperimage->setVisible(false);
+
 	m_plot_group.clear();	// Memory leak?  Destroy groups first?
 }
 
@@ -671,6 +729,7 @@ void QtGnuplotScene::setBrushStyle(int style)
 	int fillstyle = style & 0xf;
 
 	m_currentBrush.setStyle(Qt::SolidPattern);
+	m_currentFillStyle = fillstyle;
 
 	QColor color = m_currentPen.color();
 
@@ -689,7 +748,6 @@ void QtGnuplotScene::setBrushStyle(int style)
 		}
 	}
 	else if ((fillstyle == FS_TRANSPARENT_PATTERN) || (fillstyle == FS_PATTERN))
-		/// @todo color & transparent. See other terms
 		m_currentBrush.setStyle(QtGnuplot::brushes[abs(fillpar) % 8]);
 	else if (fillstyle == FS_EMPTY) // fill with background plot->color
 		color = m_widget->backgroundColor();
@@ -703,11 +761,11 @@ void QtGnuplotScene::positionText(QGraphicsItem* item, const QPoint& point)
 
 	double cx = 0.;
 	double cy = (item->boundingRect().bottom() + item->boundingRect().top())/2.;
-	if (m_textAlignment == Qt::AlignLeft)
+	if (m_textAlignment & Qt::AlignLeft)
 		cx = item->boundingRect().left();
-	else if (m_textAlignment == Qt::AlignRight)
+	else if (m_textAlignment & Qt::AlignRight)
 		cx = item->boundingRect().right();
-	else if (m_textAlignment == Qt::AlignCenter)
+	else if (m_textAlignment & Qt::AlignCenter)
 		cx = (item->boundingRect().right() + item->boundingRect().left())/2.;
 
 	item->setTransformOriginPoint(cx, cy);
@@ -773,21 +831,25 @@ void QtGnuplotScene::mouseMoveEvent(QGraphicsSceneMouseEvent* event)
 	// Mousing for inactive widgets
 	if (!m_widget->isActive())
 	{
-		m_lineTo->hide();
-		QString status;
-		if (m_axisValid[0])
-			status += QString("x = ")
-			+ QString::number(sceneToGraph(0,event->scenePos().x()));
-		if (m_axisValid[1])
-			status += QString(" y = ")
-			+ QString::number(sceneToGraph(1,event->scenePos().y()));
-		if (m_axisValid[2])
-			status += QString(" x2 = ")
-			+ QString::number(sceneToGraph(2,event->scenePos().x()));
-		if (m_axisValid[3])
-			status += QString(" y2 = ")
-			+ QString::number(sceneToGraph(3,event->scenePos().y()));
-		m_widget->setStatusText(status);
+		if (m_axisValid[4])	{
+			; // 3D plot - no mouse coordinate update
+		} else {
+			m_lineTo->hide();
+			QString status;
+			if (m_axisValid[0])
+				status += QString("x = ")
+				+ QString::number(sceneToGraph(0,event->scenePos().x()));
+			if (m_axisValid[1])
+				status += QString(" y = ")
+				+ QString::number(sceneToGraph(1,event->scenePos().y()));
+			if (m_axisValid[2])
+				status += QString(" x2 = ")
+				+ QString::number(sceneToGraph(2,event->scenePos().x()));
+			if (m_axisValid[3])
+				status += QString(" y2 = ")
+				+ QString::number(sceneToGraph(3,event->scenePos().y()));
+			m_widget->setStatusText(status);
+		}
 		QGraphicsScene::mouseMoveEvent(event);
 		return;
 	}
@@ -805,6 +867,7 @@ void QtGnuplotScene::mouseMoveEvent(QGraphicsSceneMouseEvent* event)
 	// The first item in m_hypertextList is always a background rectangle for the text
 	int i = m_hypertextList.count();
 	bool hit = false;
+	m_selectedHypertext.clear();
 	while (i-- > 1) {
 		if (!hit && ((m_hypertextList[i]->pos() - m_textOffset) 
 				- m_lastMousePos).manhattanLength() <= 5) {
@@ -813,8 +876,26 @@ void QtGnuplotScene::mouseMoveEvent(QGraphicsSceneMouseEvent* event)
 			((QGraphicsRectItem *)m_hypertextList[0])->setRect(m_hypertextList[i]->boundingRect());
 			m_hypertextList[0]->setPos(m_hypertextList[i]->pos());
 			m_hypertextList[0]->setZValue(m_hypertextList[i]->zValue()-1);
+
+			// Special hypertext "image{(xsize,ysize)}:filename" 
+			QString current_text = ((QGraphicsTextItem *)(m_hypertextList[i]))->toPlainText();
+			if (current_text.startsWith("image")) {
+				int sep = current_text.indexOf(":");
+				QString imagename = current_text.mid(sep+1);
+				sep = imagename.indexOf("\n");
+				if (sep > 0)
+					imagename.truncate(sep);
+				m_hyperimage->setPixmap(QPixmap(imagename));
+				m_hyperimage->setVisible(true);
+				break;
+			} else {
+				// If there is a mouse click we will push this to the clipboard
+				m_selectedHypertext = current_text;
+			}
+
 		} else {
 			m_hypertextList[i]->setVisible(false);
+			m_hyperimage->setVisible(false);
 		}
 	}
 	m_hypertextList[0]->setVisible(hit);
@@ -833,34 +914,35 @@ void QtGnuplotScene::mouseReleaseEvent(QGraphicsSceneMouseEvent* event)
 	else if (event->button()== Qt::MidButton)   button = 2;
 	else if (event->button()== Qt::RightButton) button = 3;
 
-	// FIXME: If the press/release events get out of order or if we see a very
-	// fast double-click then the program errors out during event processing.
-	// Unfortunately I have not been able to reliably filter out these spurious events
-	// by setting a minimum time window.  Maybe an explicit interlock would work?
-	qint64 time = 0;
+	qint64 time = 301;	/* Only used the first time in, when timer not yet running */
 	if (m_watches[button].isValid())
 		time = m_watches[button].elapsed();
-	if (time > 300) {
-		m_eventHandler->postTermEvent(GE_buttonrelease,
-			int(event->scenePos().x()), int(event->scenePos().y()), button, time, m_widget);
-	}
+	m_eventHandler->postTermEvent(GE_buttonrelease,
+		int(event->scenePos().x()), int(event->scenePos().y()), button, time, m_widget);
 	m_watches[button].start();
 
-	/* Check for click in one of the keysample boxes */
-	int n = m_key_boxes.count();
-	for (int i = 0; i < n; i++) {
-		if (m_key_boxes[i].contains(m_lastMousePos)) {
-			if (m_plot_group[i]->isVisible()) {
-				m_plot_group[i]->setVisible(false);
-				m_key_boxes[i].setHidden(true);
-			} else {
-				m_plot_group[i]->setVisible(true);
-				m_key_boxes[i].setHidden(false);
-			}
-			break;
-		}
+	/* Check for left click in one of the keysample boxes */
+	if (button == 1) {
+	    int n = m_key_boxes.count();
+	    for (int i = 0; i < n; i++) {
+		    if (m_key_boxes[i].contains(m_lastMousePos)) {
+			    if (m_plot_group[i]->isVisible()) {
+				    m_plot_group[i]->setVisible(false);
+				    m_key_boxes[i].setHidden(true);
+			    } else {
+				    m_plot_group[i]->setVisible(true);
+				    m_key_boxes[i].setHidden(false);
+			    }
+			    m_preserve_visibility = true;
+			    break;
+		    }
+	    }
 	}
 
+	// Supposedly !isEmpty indicates the mouse is hovering over a hypertext anchor
+	if ((button == 1) && !m_selectedHypertext.isEmpty())
+		QApplication::clipboard()->setText(m_selectedHypertext);
+
 	QGraphicsScene::mouseReleaseEvent(event);
 }
 
@@ -903,12 +985,19 @@ void QtGnuplotScene::keyPressEvent(QKeyEvent* event)
 			case Qt::Key_F4       : key = GP_KP_F4       ; break;
 			case Qt::Key_Insert   : key = GP_KP_Insert   ; break;
 			case Qt::Key_End      : key = GP_KP_End      ; break;
+			case Qt::Key_Home     : key = GP_KP_Home     ; break;
+#ifdef __APPLE__
+			case Qt::Key_Down     : key = GP_Down        ; break;
+			case Qt::Key_Left     : key = GP_Left        ; break;
+			case Qt::Key_Right    : key = GP_Right       ; break;
+			case Qt::Key_Up       : key = GP_Up          ; break;
+#else
 			case Qt::Key_Down     : key = GP_KP_Down     ; break;
-			case Qt::Key_PageDown : key = GP_KP_Page_Down; break;
 			case Qt::Key_Left     : key = GP_KP_Left     ; break;
 			case Qt::Key_Right    : key = GP_KP_Right    ; break;
-			case Qt::Key_Home     : key = GP_KP_Home     ; break;
 			case Qt::Key_Up       : key = GP_KP_Up       ; break;
+#endif
+			case Qt::Key_PageDown : key = GP_KP_Page_Down; break;
 			case Qt::Key_PageUp   : key = GP_KP_Page_Up  ; break;
 			case Qt::Key_Delete   : key = GP_KP_Delete   ; break;
 			case Qt::Key_Equal    : key = GP_KP_Equal    ; break;
@@ -930,9 +1019,14 @@ void QtGnuplotScene::keyPressEvent(QKeyEvent* event)
 			case Qt::Key_9        : key = GP_KP_9        ; break;
 		}
 	// ASCII keys
-	else if ((event->key() <= 0xff) && (!event->text().isEmpty()))
-		// event->key() does not respect the case
-		key = event->text()[0].toLatin1();
+	else if (event->key() <= 0xff) {
+		key = event->key();
+		// the core code is supposed handle upper/lower case by
+		// inspecting the Shift modifier, but currently that does not work
+		// so instead we check here for lowercase and pass it as text.
+		if (islower((event->text()[0].toLatin1())))
+			key = event->text()[0].toLatin1();
+	}
 	// Special keys
 	else
 		switch (event->key())
@@ -968,6 +1062,10 @@ void QtGnuplotScene::keyPressEvent(QKeyEvent* event)
 			case Qt::Key_F12        : key = GP_F12        ; break;
 		}
 
+	// The <tab> key is special.  We will catch it on keyRelease.
+	if (key == GP_Tab)
+		key = 0;
+
 	if (key >= 0)
 		live = m_eventHandler->postTermEvent(GE_keypress,
 			int(m_lastMousePos.x()), int(m_lastMousePos.y()), key, 0, m_widget);
@@ -998,6 +1096,24 @@ void QtGnuplotScene::keyPressEvent(QKeyEvent* event)
 	QGraphicsScene::keyPressEvent(event);
 }
 
+/*
+ * Qt swallows the keyPress events from <tab> because it is used to
+ * cycle focus among a set of widgets. The documented methods for bypassing
+ * this focus-related theft are beyond my comprehension.  So instead we
+ * catch the key release event (for <tab> only) and pretend it was a press.
+ */
+void QtGnuplotScene::keyReleaseEvent(QKeyEvent* event)
+{
+	if (Qt::Key_Tab == event->key()) {
+		updateModifiers();
+		m_eventHandler->postTermEvent(GE_keypress,
+			int(m_lastMousePos.x()), int(m_lastMousePos.y()),
+			GP_Tab, 0, m_widget);
+	}
+
+	QGraphicsScene::keyReleaseEvent(event);
+}
+
 double QtGnuplotScene::sceneToGraph(int axis, double coord) const
 {
 	if (m_axisScale[axis] == 0.)
